#include <directfb.h>

#include <stdio.h>

static IDirectFB *dfb;
static IDirectFBDisplayLayer *layer;
static IDirectFBSurface *background;

static IDirectFBFont *font;

static IDirectFBWindow *win1;
static IDirectFBSurface *win1_surface;

static IDirectFBEventBuffer *event;

/**
 * 画像などのリソースデータディレクトリを指定しています。
 * このサンプルでは、DirectFB-examplesのものを使用します。
 **/
#define DATADIR "/usr/local/share/directfb-examples/"

/**
 * 簡単なエラーチェックマクロ。
 **/
#define DFBCHECK(x...)                                              \
({                                                                  \
        if (x != DFB_OK) {                                          \
                fprintf(stderr, "%s <%d>:\n\t", __FILE__, __LINE__);\
                DirectFBErrorFatal(#x, x);                          \
        }                                                           \
})

int
main(int argc, char **argv)
{
	static int quit = 0;

        DFBGraphicsDeviceDescription desc;
        DFBDisplayLayerConfig config;

	/**
	 * DirectFBの初期化を行い、スーパーインタフェースを作成します。
	 * また、プライマリレイヤを取得し設定を行います。
	 **/
        DFBCHECK(DirectFBInit(&argc, &argv));
        DFBCHECK(DirectFBCreate(&dfb));

        DFBCHECK(dfb->GetDeviceDescription(dfb, &desc));

        DFBCHECK(dfb->GetDisplayLayer(dfb, DLID_PRIMARY, &layer));

        DFBCHECK(layer->SetCooperativeLevel(layer, DLSCL_ADMINISTRATIVE));

        if (!((desc.blitting_flags & DSBLIT_BLEND_ALPHACHANNEL) &&
              (desc.blitting_flags & DSBLIT_BLEND_COLORALPHA))) {
		config.flags = DLCONF_BUFFERMODE;
                config.buffermode = DLBM_BACKSYSTEM;

                DFBCHECK(layer->SetConfiguration(layer, &config));
        }
        DFBCHECK(layer->GetConfiguration(layer, &config));

	/**
	 * フォントを読み込みます。
	 **/
	{
		DFBFontDescription desc;
		desc.flags = DFDESC_HEIGHT;
		desc.height = config.height / 10;

		DFBCHECK(dfb->CreateFont(dfb,
					 DATADIR"fonts/decker.ttf",
					 &desc, &font));
	}

	/**
	 * カーソル表示を有効にします。
   	 **/
        DFBCHECK(layer->EnableCursor(layer, 1));

	/**
	 * バックグラウンド用のサーフェイスを作成し、赤色で塗りつぶします。
	 **/
	{
		DFBSurfaceDescription desc;
		desc.flags = DSDESC_WIDTH | DSDESC_HEIGHT;
		desc.width = config.width;
		desc.height = config.height;
		DFBCHECK(dfb->CreateSurface(dfb, &desc, &background));

		DFBCHECK(background->SetColor(background,
					      0xff, 0x00, 0x00, 0xff));
		DFBCHECK(background->FillRectangle(background,
						   0, 0,
						   desc.width, desc.height));
	}

	/**
	 * バックグラウンドに文字列を描画します。
	 **/
	{
		DFBCHECK(background->SetFont(background, font));
		DFBCHECK(background->SetColor(background,
					      0xff, 0xff, 0xff, 0xff));
		DFBCHECK(background->DrawString(background,
						"DFB_TEST", -1, 10, 10,
						DSTF_LEFT | DSTF_TOP));
	}

	/**
	 * プライマリレイヤにバックラウンドサーフェイスを登録します。
	 **/
	{
		DFBCHECK(layer->SetBackgroundImage(layer, background));
		DFBCHECK(layer->SetBackgroundMode(layer, DLBM_IMAGE));
	}

	/**
	 * プライマリレイヤにウィンドウを作成し、そのウィンドウに
	 * 画像を描画します。
	 **/
	{
		DFBWindowDescription desc;
		IDirectFBImageProvider *provider;

		desc.flags = (DWDESC_POSX | DWDESC_POSY |
			      DWDESC_WIDTH | DWDESC_HEIGHT | DWDESC_CAPS);
		desc.posx = 100;
		desc.posy = 100;
		desc.width = config.width - 200;
		desc.height = config.height - 200;
		desc.caps = DWCAPS_ALPHACHANNEL;
	  
		DFBCHECK(layer->CreateWindow(layer, &desc, &win1));
		DFBCHECK(win1->GetSurface(win1, &win1_surface));

		DFBCHECK(dfb->CreateImageProvider(dfb,
						  DATADIR"/dfblogo.png",
						  &provider));
		DFBCHECK(provider->RenderTo(provider, win1_surface, NULL));
		
		DFBCHECK(provider->Release(provider));
		
		DFBCHECK(win1->SetOpacity(win1, 0xFF));
	}

	/**
	 * 作成したウィンドウのイベントを取得できるように、
	 * イベントバッファを作成します。
	 **/
	DFBCHECK(win1->CreateEventBuffer(win1, &event));

	/**
	 * イベント待ちループ。
	 * ウィンドウ上でボタン押下イベントを検出した場合に、
	 * ループから抜けます。
	 **/
	while (!quit) {
		DFBEvent evt;
		DFBCHECK(event->WaitForEvent(event));

		while (event->HasEvent(event) == DFB_OK) {
			DFBCHECK(event->GetEvent(event, &evt));
		  
			if (evt.clazz == DFEC_WINDOW)
				if (evt.window.type == DWET_BUTTONDOWN)
					quit = 1;
		}
	}

	/**
	 * 各DirectFBのインタフェースを開放します。
	 **/
	event->Release(event);
	win1_surface->Release(win1_surface);
	win1->Release(win1);

	font->Release(font);
	background->Release(background);

        layer->Release(layer);
        dfb->Release(dfb);

	return 0;
}
